@@ -45,7 +45,7 @@ import GHC.Types.Unique.Set
 import GHC.Types.Unique.FM
 import GHC.Types.Unique
 import GHC.Builtin.Uniques
-import GHC.Utils.Monad.State
+import GHC.Utils.Monad.State.Strict
 import GHC.Utils.Outputable
 import GHC.Utils.Panic
 import GHC.Platform
@@ -516,9 +516,6 @@ instance Uniquable Store where
         | RegReal (RealRegSingle i)     <- r
         = mkRegSingleUnique i
 
-        | RegReal (RealRegPair r1 r2)   <- r
-        = mkRegPairUnique (r1 * 65535 + r2)
-
         | otherwise
         = error $ "RegSpillClean.getUnique: found virtual reg during spill clean,"
                 ++ "only real regs expected."
