@@ -1,4 +1,4 @@
-{-# LANGUAGE CPP                 #-}
+
 {-# LANGUAGE ScopedTypeVariables #-}
 {-# LANGUAGE MultiWayIf          #-}
 {-# LANGUAGE TypeApplications    #-}
@@ -24,14 +24,13 @@ module GHC.HsToCore.Match.Literal
    )
 where
 
-#include "HsVersions.h"
-
 import GHC.Prelude
 import GHC.Platform
 
 import {-# SOURCE #-} GHC.HsToCore.Match ( match )
 import {-# SOURCE #-} GHC.HsToCore.Expr  ( dsExpr, dsSyntaxExpr )
 
+import GHC.HsToCore.Errors.Types
 import GHC.HsToCore.Monad
 import GHC.HsToCore.Utils
 
@@ -42,6 +41,7 @@ import GHC.Types.SourceText
 import GHC.Core
 import GHC.Core.Make
 import GHC.Core.TyCon
+import GHC.Core.Reduction ( Reduction(..) )
 import GHC.Core.DataCon
 import GHC.Tc.Utils.Zonk ( shortCutLit )
 import GHC.Tc.Utils.TcType
@@ -56,8 +56,8 @@ import GHC.Utils.Outputable as Outputable
 import GHC.Driver.Session
 import GHC.Utils.Misc
 import GHC.Utils.Panic
+import GHC.Utils.Panic.Plain
 import GHC.Data.FastString
-import qualified GHC.LanguageExtensions as LangExt
 import GHC.Core.FamInstEnv ( FamInstEnvs, normaliseType )
 
 import Control.Monad
@@ -108,7 +108,7 @@ dsLit l = do
     HsDoublePrim _ fl -> return (Lit (LitDouble (rationalFromFractionalLit fl)))
     HsChar _ c       -> return (mkCharExpr c)
     HsString _ str   -> mkStringExprFS str
-    HsInteger _ i _  -> return (mkIntegerExpr i)
+    HsInteger _ i _  -> return (mkIntegerExpr platform i)
     HsInt _ i        -> return (mkIntExpr platform (il_value i))
     HsRat _ fl ty    -> dsFractionalLitToRational fl ty
 
@@ -199,15 +199,17 @@ dsFractionalLitToRational :: FractionalLit -> Type -> DsM CoreExpr
 dsFractionalLitToRational fl@FL{ fl_signi = signi, fl_exp = exp, fl_exp_base = base } ty
   -- We compute "small" rationals here and now
   | abs exp <= 100
-  = let !val   = rationalFromFractionalLit fl
-        !num   = mkIntegerExpr (numerator val)
-        !denom = mkIntegerExpr (denominator val)
+  = do
+    platform <- targetPlatform <$> getDynFlags
+    let !val   = rationalFromFractionalLit fl
+        !num   = mkIntegerExpr platform (numerator val)
+        !denom = mkIntegerExpr platform (denominator val)
         (ratio_data_con, integer_ty)
             = case tcSplitTyConApp ty of
-                    (tycon, [i_ty]) -> ASSERT(isIntegerTy i_ty && tycon `hasKey` ratioTyConKey)
+                    (tycon, [i_ty]) -> assert (isIntegerTy i_ty && tycon `hasKey` ratioTyConKey)
                                        (head (tyConDataCons tycon), i_ty)
                     x -> pprPanic "dsLit" (ppr x)
-    in return $! (mkCoreConApps ratio_data_con [Type integer_ty, num, denom])
+    return $! (mkCoreConApps ratio_data_con [Type integer_ty, num, denom])
   -- Large rationals will be computed at runtime.
   | otherwise
   = do
@@ -216,22 +218,23 @@ dsFractionalLitToRational fl@FL{ fl_signi = signi, fl_exp = exp, fl_exp_base = b
                              Base10 -> mkRationalBase10Name
       mkRational <- dsLookupGlobalId mkRationalName
       litR <- dsRational signi
-      let litE = mkIntegerExpr exp
+      platform <- targetPlatform <$> getDynFlags
+      let litE = mkIntegerExpr platform exp
       return (mkCoreApps (Var mkRational) [litR, litE])
 
 dsRational :: Rational -> DsM CoreExpr
 dsRational (n :% d) = do
+  platform <- targetPlatform <$> getDynFlags
   dcn <- dsLookupDataCon ratioDataConName
-  let cn = mkIntegerExpr n
-  let dn = mkIntegerExpr d
+  let cn = mkIntegerExpr platform n
+  let dn = mkIntegerExpr platform d
   return $ mkCoreConApps dcn [Type integerTy, cn, dn]
 
 
 dsOverLit :: HsOverLit GhcTc -> DsM CoreExpr
 -- ^ Post-typechecker, the 'HsExpr' field of an 'OverLit' contains
 -- (an expression for) the literal value itself.
-dsOverLit (OverLit { ol_val = val, ol_ext = OverLitTc rebindable ty
-                   , ol_witness = witness }) = do
+dsOverLit (OverLit { ol_val = val, ol_ext = OverLitTc rebindable witness ty }) = do
   dflags <- getDynFlags
   let platform = targetPlatform dflags
   case shortCutLit platform val ty of
@@ -264,10 +267,7 @@ warnAboutIdentities dflags conv_fn type_of_conv
   , idName conv_fn `elem` conversionNames
   , Just (_, arg_ty, res_ty) <- splitFunTy_maybe type_of_conv
   , arg_ty `eqType` res_ty  -- So we are converting  ty -> ty
-  = warnDs (Reason Opt_WarnIdentities)
-           (vcat [ text "Call of" <+> ppr conv_fn <+> dcolon <+> ppr type_of_conv
-                 , nest 2 $ text "can probably be omitted"
-           ])
+  = diagnosticDs (DsIdentitiesFound conv_fn type_of_conv)
 warnAboutIdentities _ _ _ = return ()
 
 conversionNames :: [Name]
@@ -348,37 +348,13 @@ warnAboutOverflowedLiterals dflags lit
     checkPositive :: Integer -> Name -> DsM ()
     checkPositive i tc
       = when (i < 0) $
-        warnDs (Reason Opt_WarnOverflowedLiterals)
-               (vcat [ text "Literal" <+> integer i
-                       <+> text "is negative but" <+> ppr tc
-                       <+> ptext (sLit "only supports positive numbers")
-                     ])
+        diagnosticDs (DsOverflowedLiterals i tc Nothing (negLiteralExtEnabled dflags))
 
     check i tc minB maxB
       = when (i < minB || i > maxB) $
-        warnDs (Reason Opt_WarnOverflowedLiterals)
-               (vcat [ text "Literal" <+> integer i
-                       <+> text "is out of the" <+> ppr tc <+> ptext (sLit "range")
-                       <+> integer minB <> text ".." <> integer maxB
-                     , sug ])
+        diagnosticDs (DsOverflowedLiterals i tc bounds (negLiteralExtEnabled dflags))
       where
-        sug | minB == -i   -- Note [Suggest NegativeLiterals]
-            , i > 0
-            , not (xopt LangExt.NegativeLiterals dflags)
-            = text "If you are trying to write a large negative literal, use NegativeLiterals"
-            | otherwise = Outputable.empty
-
-{-
-Note [Suggest NegativeLiterals]
-~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-If you write
-  x :: Int8
-  x = -128
-it'll parse as (negate 128), and overflow.  In this case, suggest NegativeLiterals.
-We get an erroneous suggestion for
-  x = 128
-but perhaps that does not matter too much.
--}
+        bounds = Just (MinBound minB, MaxBound maxB)
 
 warnAboutEmptyEnumerations :: FamInstEnvs -> DynFlags -> LHsExpr GhcTc
                            -> Maybe (LHsExpr GhcTc)
@@ -441,19 +417,20 @@ warnAboutEmptyEnumerations fam_envs dflags fromExpr mThnExpr toExpr
 
   | otherwise = return ()
   where
-    raiseWarning = warnDs (Reason Opt_WarnEmptyEnumerations) (text "Enumeration is empty")
+    raiseWarning =
+      diagnosticDs DsEmptyEnumeration
 
 getLHsIntegralLit :: LHsExpr GhcTc -> Maybe (Integer, Type)
 -- ^ See if the expression is an 'Integral' literal.
 getLHsIntegralLit (L _ e) = go e
   where
-    go (HsPar _ e)            = getLHsIntegralLit e
+    go (HsPar _ _ e _)        = getLHsIntegralLit e
     go (HsOverLit _ over_lit) = getIntegralLit over_lit
     go (HsLit _ lit)          = getSimpleIntegralLit lit
 
     -- Remember to look through automatically-added tick-boxes! (#8384)
-    go (HsTick _ _ e)         = getLHsIntegralLit e
-    go (HsBinTick _ _ _ e)    = getLHsIntegralLit e
+    go (XExpr (HsTick _ e))       = getLHsIntegralLit e
+    go (XExpr (HsBinTick _ _ e))  = getLHsIntegralLit e
 
     -- The literal might be wrapped in a case with -XOverloadedLists
     go (XExpr (WrapExpr (HsWrap _ e))) = go e
@@ -462,7 +439,7 @@ getLHsIntegralLit (L _ e) = go e
 -- | If 'Integral', extract the value and type of the overloaded literal.
 -- See Note [Literals and the OverloadedLists extension]
 getIntegralLit :: HsOverLit GhcTc -> Maybe (Integer, Type)
-getIntegralLit (OverLit { ol_val = HsIntegral i, ol_ext = OverLitTc _ ty })
+getIntegralLit (OverLit { ol_val = HsIntegral i, ol_ext = OverLitTc { ol_type = ty } })
   = Just (il_value i, ty)
 getIntegralLit _ = Nothing
 
@@ -478,10 +455,10 @@ getSimpleIntegralLit _ = Nothing
 
 -- | Extract the Char if the expression is a Char literal.
 getLHsCharLit :: LHsExpr GhcTc -> Maybe Char
-getLHsCharLit (L _ (HsPar _ e))            = getLHsCharLit e
-getLHsCharLit (L _ (HsTick _ _ e))         = getLHsCharLit e
-getLHsCharLit (L _ (HsBinTick _ _ _ e))    = getLHsCharLit e
+getLHsCharLit (L _ (HsPar _ _ e _))        = getLHsCharLit e
 getLHsCharLit (L _ (HsLit _ (HsChar _ c))) = Just c
+getLHsCharLit (L _ (XExpr (HsTick _ e)))         = getLHsCharLit e
+getLHsCharLit (L _ (XExpr (HsBinTick _ _ e)))    = getLHsCharLit e
 getLHsCharLit _ = Nothing
 
 -- | Convert a pair (Integer, Type) to (Integer, Name) after eventually
@@ -493,7 +470,9 @@ getNormalisedTyconName fam_envs (i,ty)
     | otherwise = Nothing
   where
     normaliseNominal :: FamInstEnvs -> Type -> Type
-    normaliseNominal fam_envs ty = snd $ normaliseType fam_envs Nominal ty
+    normaliseNominal fam_envs ty
+      = reductionReducedType
+      $ normaliseType fam_envs Nominal ty
 
 -- | Convert a pair (Integer, Type) to (Integer, Name) without normalising
 -- the type
@@ -527,9 +506,9 @@ to convert it to the needed type `Word8`.
 
 tidyLitPat :: HsLit GhcTc -> Pat GhcTc
 -- Result has only the following HsLits:
---      HsIntPrim, HsWordPrim, HsCharPrim, HsFloatPrim
---      HsDoublePrim, HsStringPrim, HsString
---  * HsInteger, HsRat, HsInt can't show up in LitPats
+--      HsIntPrim, HsWordPrim, HsCharPrim, HsString
+--  * HsInteger, HsRat, HsInt, as well as HsStringPrim,
+--    HsFloatPrim and HsDoublePrim can't show up in LitPats
 --  * We get rid of HsChar right here
 tidyLitPat (HsChar src c) = unLoc (mkCharLitPat src c)
 tidyLitPat (HsString src s)
@@ -545,7 +524,7 @@ tidyLitPat lit = LitPat noExtField lit
 tidyNPat :: HsOverLit GhcTc -> Maybe (SyntaxExpr GhcTc) -> SyntaxExpr GhcTc
          -> Type
          -> Pat GhcTc
-tidyNPat (OverLit (OverLitTc False ty) val _) mb_neg _eq outer_ty
+tidyNPat (OverLit (OverLitTc False _ ty) val) mb_neg _eq outer_ty
         -- False: Take short cuts only if the literal is not using rebindable syntax
         --
         -- Once that is settled, look for cases where the type of the
@@ -589,7 +568,7 @@ tidyNPat (OverLit (OverLitTc False ty) val _) mb_neg _eq outer_ty
                    _ -> Nothing
 
 tidyNPat over_lit mb_neg eq outer_ty
-  = NPat outer_ty (noLoc over_lit) mb_neg eq
+  = NPat outer_ty (noLocA over_lit) mb_neg eq
 
 {-
 ************************************************************************
