@@ -1,4 +1,4 @@
-{-# LANGUAGE CPP                 #-}
+
 {-# LANGUAGE LambdaCase          #-}
 {-# LANGUAGE ScopedTypeVariables #-}
 {-# LANGUAGE ViewPatterns        #-}
@@ -23,23 +23,20 @@ Authors: George Karachalias <george.karachalias@cs.kuleuven.be>
 module GHC.HsToCore.Pmc.Solver (
 
         Nabla, Nablas(..), initNablas,
-        lookupRefuts, lookupSolution,
 
         PhiCt(..), PhiCts,
         addPhiCtNablas,
         addPhiCtsNablas,
 
         isInhabited,
-        generateInhabitingPatterns
+        generateInhabitingPatterns, GenerateInhabitingPatternsMode(..)
 
     ) where
 
-#include "HsVersions.h"
-
 import GHC.Prelude
 
 import GHC.HsToCore.Pmc.Types
-import GHC.HsToCore.Pmc.Utils (tracePm, mkPmId)
+import GHC.HsToCore.Pmc.Utils (tracePm, traceWhenFailPm, mkPmId)
 
 import GHC.Driver.Session
 import GHC.Driver.Config
@@ -47,7 +44,9 @@ import GHC.Utils.Outputable
 import GHC.Utils.Misc
 import GHC.Utils.Monad (allM)
 import GHC.Utils.Panic
+import GHC.Utils.Panic.Plain
 import GHC.Data.Bag
+import GHC.Types.Basic (Levity(..))
 import GHC.Types.CompleteMatch
 import GHC.Types.Unique.Set
 import GHC.Types.Unique.DSet
@@ -72,6 +71,7 @@ import GHC.Core.DataCon
 import GHC.Core.PatSyn
 import GHC.Core.TyCon
 import GHC.Core.TyCon.RecWalk
+import GHC.Builtin.Names
 import GHC.Builtin.Types
 import GHC.Builtin.Types.Prim (tYPETyCon)
 import GHC.Core.TyCo.Rep
@@ -80,6 +80,7 @@ import GHC.Core.Type
 import GHC.Tc.Solver   (tcNormalise, tcCheckGivens, tcCheckWanteds)
 import GHC.Core.Unify    (tcMatchTy)
 import GHC.Core.Coercion
+import GHC.Core.Reduction
 import GHC.HsToCore.Monad hiding (foldlM)
 import GHC.Tc.Instance.Family
 import GHC.Core.FamInstEnv
@@ -96,6 +97,9 @@ import Data.List     (sortBy, find)
 import qualified Data.List.NonEmpty as NE
 import Data.Ord      (comparing)
 
+import GHC.Utils.Trace
+_ = pprTrace -- to silence unused import warnings
+
 --
 -- * Main exports
 --
@@ -143,7 +147,7 @@ updRcm f (RCM vanilla pragmas)
 -- Ex.: @vanillaCompleteMatchTC 'Maybe' ==> Just ("Maybe", {'Just','Nothing'})@
 vanillaCompleteMatchTC :: TyCon -> Maybe CompleteMatch
 vanillaCompleteMatchTC tc =
-  let -- | TYPE acts like an empty data type on the term-level (#14086), but
+  let -- TYPE acts like an empty data type on the term-level (#14086), but
       -- it is a PrimTyCon, so tyConDataCons_maybe returns Nothing. Hence a
       -- special case.
       mb_dcs | tc == tYPETyCon = Just []
@@ -169,7 +173,7 @@ addConLikeMatches (PatSynCon _)    rcm = addCompleteMatches rcm
 addTyConMatches :: TyCon -> ResidualCompleteMatches -> DsM ResidualCompleteMatches
 addTyConMatches tc rcm = add_tc_match <$> addCompleteMatches rcm
   where
-    -- | Add the vanilla COMPLETE set from the data defn, if any. But only if
+    -- Add the vanilla COMPLETE set from the data defn, if any. But only if
     -- it's not already present.
     add_tc_match rcm
       = rcm{rcm_vanilla = rcm_vanilla rcm <|> vanillaCompleteMatchTC tc}
@@ -326,23 +330,23 @@ pmTopNormaliseType :: TyState -> Type -> DsM TopNormaliseTypeResult
 -- NB: Normalisation can potentially change kinds, if the head of the type
 -- is a type family with a variable result kind. I (Richard E) can't think
 -- of a way to cause trouble here, though.
-pmTopNormaliseType (TySt _ inert) typ
-  = do env <- dsGetFamInstEnvs
-       tracePm "normalise" (ppr typ)
-       -- Before proceeding, we chuck typ into the constraint solver, in case
-       -- solving for given equalities may reduce typ some. See
-       -- "Wrinkle: local equalities" in Note [Type normalisation].
-       typ' <- initTcDsForSolver $ tcNormalise inert typ
-       -- Now we look with topNormaliseTypeX through type and data family
-       -- applications and newtypes, which tcNormalise does not do.
-       -- See also 'TopNormaliseTypeResult'.
-       pure $ case topNormaliseTypeX (stepper env) comb typ' of
-         Nothing                -> NormalisedByConstraints typ'
-         Just ((ty_f,tm_f), ty) -> HadRedexes src_ty newtype_dcs core_ty
-           where
-             src_ty = eq_src_ty ty (typ' : ty_f [ty])
-             newtype_dcs = tm_f []
-             core_ty = ty
+pmTopNormaliseType (TySt _ inert) typ = {-# SCC "pmTopNormaliseType" #-} do
+  env <- dsGetFamInstEnvs
+  tracePm "normalise" (ppr typ)
+  -- Before proceeding, we chuck typ into the constraint solver, in case
+  -- solving for given equalities may reduce typ some. See
+  -- "Wrinkle: local equalities" in Note [Type normalisation].
+  typ' <- initTcDsForSolver $ tcNormalise inert typ
+  -- Now we look with topNormaliseTypeX through type and data family
+  -- applications and newtypes, which tcNormalise does not do.
+  -- See also 'TopNormaliseTypeResult'.
+  pure $ case topNormaliseTypeX (stepper env) comb typ' of
+    Nothing                -> NormalisedByConstraints typ'
+    Just ((ty_f,tm_f), ty) -> HadRedexes src_ty newtype_dcs core_ty
+      where
+        src_ty = eq_src_ty ty (typ' : ty_f [ty])
+        newtype_dcs = tm_f []
+        core_ty = ty
   where
     -- Find the first type in the sequence of rewrites that is a data type,
     -- newtype, or a data family application (not the representation tycon!).
@@ -378,8 +382,9 @@ pmTopNormaliseType (TySt _ inert) typ
     tyFamStepper :: FamInstEnvs -> NormaliseStepper ([Type] -> [Type], a -> a)
     tyFamStepper env rec_nts tc tys  -- Try to step a type/data family
       = case topReduceTyFamApp_maybe env tc tys of
-          Just (_, rhs, _) -> NS_Step rec_nts rhs ((rhs:), id)
-          _                -> NS_Done
+          Just (HetReduction (Reduction _ rhs) _)
+            -> NS_Step rec_nts rhs ((rhs:), id)
+          _ -> NS_Done
 
 -- | Returns 'True' if the argument 'Type' is a fully saturated application of
 -- a closed type constructor.
@@ -397,7 +402,7 @@ pmIsClosedType ty
   = case splitTyConApp_maybe ty of
       Just (tc, ty_args)
              | is_algebraic_like tc && not (isFamilyTyCon tc)
-             -> ASSERT2( ty_args `lengthIs` tyConArity tc, ppr ty ) True
+             -> assertPpr (ty_args `lengthIs` tyConArity tc) (ppr ty) True
       _other -> False
   where
     -- This returns True for TyCons which /act like/ algebraic types.
@@ -511,61 +516,9 @@ inhabitationCandidates, we'll mistakenly conclude that `f` is non-exhaustive.
 In order to avoid this pitfall, we need to normalise the type passed to
 pmTopNormaliseType, using the constraint solver to solve for any local
 equalities (such as i ~ Int) that may be in scope.
--}
 
------------------------
--- * Looking up VarInfo
-
-emptyRCM :: ResidualCompleteMatches
-emptyRCM = RCM Nothing Nothing
-
-emptyVarInfo :: Id -> VarInfo
-emptyVarInfo x
-  = VI
-  { vi_id = x
-  , vi_pos = []
-  , vi_neg = emptyPmAltConSet
-  -- Why not set IsNotBot for unlifted type here?
-  -- Because we'd have to trigger an inhabitation test, which we can't.
-  -- See case (4) in Note [Strict fields and variables of unlifted type]
-  -- in GHC.HsToCore.Pmc.Solver
-  , vi_bot = MaybeBot
-  , vi_rcm = emptyRCM
-  }
-
-lookupVarInfo :: TmState -> Id -> VarInfo
--- (lookupVarInfo tms x) tells what we know about 'x'
-lookupVarInfo (TmSt env _ _) x = fromMaybe (emptyVarInfo x) (lookupUSDFM env x)
-
--- | Like @lookupVarInfo ts x@, but @lookupVarInfo ts x = (y, vi)@ also looks
--- through newtype constructors. We have @x ~ N1 (... (Nk y))@ such that the
--- returned @y@ doesn't have a positive newtype constructor constraint
--- associated with it (yet). The 'VarInfo' returned is that of @y@'s
--- representative.
---
--- Careful, this means that @idType x@ might be different to @idType y@, even
--- modulo type normalisation!
---
--- See also Note [Coverage checking Newtype matches].
-lookupVarInfoNT :: TmState -> Id -> (Id, VarInfo)
-lookupVarInfoNT ts x = case lookupVarInfo ts x of
-  VI{ vi_pos = as_newtype -> Just y } -> lookupVarInfoNT ts y
-  res                                 -> (x, res)
-  where
-    as_newtype = listToMaybe . mapMaybe go
-    go PACA{paca_con = PmAltConLike (RealDataCon dc), paca_ids = [y]}
-      | isNewDataCon dc = Just y
-    go _                = Nothing
-
-trvVarInfo :: Functor f => (VarInfo -> f (a, VarInfo)) -> Nabla -> Id -> f (a, Nabla)
-trvVarInfo f nabla@MkNabla{ nabla_tm_st = ts@TmSt{ts_facts = env} } x
-  = set_vi <$> f (lookupVarInfo ts x)
-  where
-    set_vi (a, vi') =
-      (a, nabla{ nabla_tm_st = ts{ ts_facts = addToUSDFM env (vi_id vi') vi' } })
-
-{- Note [Coverage checking Newtype matches]
-~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+Note [Coverage checking Newtype matches]
+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 Newtypes have quite peculiar match semantics compared to ordinary DataCons. In a
 pattern-match, they behave like a irrefutable (lazy) match, but for inhabitation
 testing purposes (e.g. at construction sites), they behave rather like a DataCon
@@ -592,28 +545,6 @@ Handling of Newtypes is also described in the Appendix of the Lower Your Guards
 where you can find the solution in a perhaps more digestible format.
 -}
 
-------------------------------------------------
--- * Exported utility functions querying 'Nabla'
-
-lookupRefuts :: Nabla -> Id -> [PmAltCon]
--- Unfortunately we need the extra bit of polymorphism and the unfortunate
--- duplication of lookupVarInfo here.
-lookupRefuts MkNabla{ nabla_tm_st = ts } x =
-  pmAltConSetElems $ vi_neg $ lookupVarInfo ts x
-
-isDataConSolution :: PmAltConApp -> Bool
-isDataConSolution PACA{paca_con = PmAltConLike (RealDataCon _)} = True
-isDataConSolution _                                             = False
-
--- @lookupSolution nabla x@ picks a single solution ('vi_pos') of @x@ from
--- possibly many, preferring 'RealDataCon' solutions whenever possible.
-lookupSolution :: Nabla -> Id -> Maybe PmAltConApp
-lookupSolution nabla x = case vi_pos (lookupVarInfo (nabla_tm_st nabla) x) of
-  []                                         -> Nothing
-  pos
-    | Just sol <- find isDataConSolution pos -> Just sol
-    | otherwise                              -> Just (head pos)
-
 -------------------------
 -- * Adding φ constraints
 --
@@ -691,13 +622,16 @@ addTyCts nabla@MkNabla{ nabla_ty_st = ty_st } new_ty_cs = do
 
 -- | Add some extra type constraints to the 'TyState'; return 'Nothing' if we
 -- find a contradiction (e.g. @Int ~ Bool@).
+--
+-- See Note [Pattern match warnings with insoluble Givens] in GHC.Tc.Solver.
 tyOracle :: TyState -> Bag PredType -> DsM (Maybe TyState)
 tyOracle ty_st@(TySt n inert) cts
   | isEmptyBag cts
   = pure (Just ty_st)
   | otherwise
-  = do { evs <- traverse nameTyCt cts
-       ; tracePm "tyOracle" (ppr cts $$ ppr inert)
+  = {-# SCC "tyOracle" #-}
+    do { evs <- traverse nameTyCt cts
+       ; tracePm "tyOracle" (ppr n $$ ppr cts $$ ppr inert)
        ; mb_new_inert <- initTcDsForSolver $ tcCheckGivens inert evs
          -- return the new inert set and increment the sequence number n
        ; return (TySt (n+1) <$> mb_new_inert) }
@@ -741,7 +675,7 @@ addPhiTmCt nabla (PhiNotBotCt x)           = addNotBotCt nabla x
 filterUnliftedFields :: PmAltCon -> [Id] -> [Id]
 filterUnliftedFields con args =
   [ arg | (arg, bang) <- zipEqual "addPhiCt" args (pmAltConImplBangs con)
-        , isBanged bang || isUnliftedType (idType arg) ]
+        , isBanged bang || typeLevity_maybe (idType arg) == Just Unlifted ]
 
 -- | Adds the constraint @x ~ ⊥@, e.g. that evaluation of a particular 'Id' @x@
 -- surely diverges. Quite similar to 'addConCt', only that it only cares about
@@ -752,11 +686,12 @@ addBotCt nabla@MkNabla{ nabla_tm_st = ts@TmSt{ ts_facts=env } } x = do
   case bot of
     IsNotBot -> mzero      -- There was x ≁ ⊥. Contradiction!
     IsBot    -> pure nabla -- There already is x ~ ⊥. Nothing left to do
-    MaybeBot ->            -- We add x ~ ⊥
+    MaybeBot               -- We add x ~ ⊥
+      | Just Unlifted <- typeLevity_maybe (idType x)
       -- Case (3) in Note [Strict fields and variables of unlifted type]
-      if isUnliftedType (idType x)
-        then mzero -- unlifted vars can never be ⊥
-        else do
+      -> mzero -- unlifted vars can never be ⊥
+      | otherwise
+      -> do
           let vi' = vi{ vi_bot = IsBot }
           pure nabla{ nabla_tm_st = ts{ts_facts = addToUSDFM env y vi' } }
 
@@ -787,7 +722,7 @@ addNotConCt nabla x nalt = do
     Just x  -> markDirty x nabla'
     Nothing -> nabla'
   where
-    -- | Update `x`'s 'VarInfo' entry. Fail ('MaybeT') if contradiction,
+    -- Update `x`'s 'VarInfo' entry. Fail ('MaybeT') if contradiction,
     -- otherwise return updated entry and `Just x'` if `x` should be marked dirty,
     -- where `x'` is the representative of `x`.
     go :: VarInfo -> MaybeT DsM (Maybe Id, VarInfo)
@@ -803,7 +738,7 @@ addNotConCt nabla x nalt = do
             -- See Note [Completeness checking with required Thetas]
             | hasRequiredTheta nalt  = neg
             | otherwise              = extendPmAltConSet neg nalt
-      MASSERT( isPmAltConMatchStrict nalt )
+      massert (isPmAltConMatchStrict nalt)
       let vi' = vi{ vi_neg = neg', vi_bot = IsNotBot }
       -- 3. Make sure there's at least one other possible constructor
       mb_rcm' <- lift (markMatched nalt rcm)
@@ -844,8 +779,6 @@ addConCt nabla@MkNabla{ nabla_tm_st = ts@TmSt{ ts_facts=env } } x alt tvs args =
     Just (PACA _con other_tvs other_args) -> do
       -- We must unify existentially bound ty vars and arguments!
       let ty_cts = equateTys (map mkTyVarTy tvs) (map mkTyVarTy other_tvs)
-      when (length args /= length other_args) $
-        lift $ tracePm "error" (ppr x <+> ppr alt <+> ppr args <+> ppr other_args)
       nabla' <- MaybeT $ addPhiCts nabla (listToBag ty_cts)
       let add_var_ct nabla (a, b) = addVarCt nabla a b
       foldlM add_var_ct nabla' $ zipEqual "addConCt" args other_args
@@ -860,7 +793,7 @@ addConCt nabla@MkNabla{ nabla_tm_st = ts@TmSt{ ts_facts=env } } x alt tvs args =
             MaybeBot -> pure (nabla_with MaybeBot)
             IsBot    -> addBotCt (nabla_with MaybeBot) y
             IsNotBot -> addNotBotCt (nabla_with MaybeBot) y
-        _ -> ASSERT( isPmAltConMatchStrict alt )
+        _ -> assert (isPmAltConMatchStrict alt )
              pure (nabla_with IsNotBot) -- strict match ==> not ⊥
 
 equateTys :: [Type] -> [Type] -> [PhiCt]
@@ -913,7 +846,7 @@ addCoreCt nabla x e = do
   -- lift $ tracePm "addCoreCt" (ppr x <+> dcolon <+> ppr (idType x) $$ ppr e $$ ppr e')
   execStateT (core_expr x e') nabla
   where
-    -- | Takes apart a 'CoreExpr' and tries to extract as much information about
+    -- Takes apart a 'CoreExpr' and tries to extract as much information about
     -- literals and constructor applications as possible.
     core_expr :: Id -> CoreExpr -> StateT Nabla (MaybeT DsM) ()
     -- TODO: Handle newtypes properly, by wrapping the expression in a DataCon
@@ -954,7 +887,7 @@ addCoreCt nabla x e = do
         -- up substituting inside a forall or lambda (i.e. seldom)
         -- so using exprFreeVars seems fine.   See MR !1647.
 
-    -- | The @e@ in @let x = e@ had no familiar form. But we can still see if
+    -- The @e@ in @let x = e@ had no familiar form. But we can still see if
     -- see if we already encountered a constraint @let y = e'@ with @e'@
     -- semantically equivalent to @e@, in which case we may add the constraint
     -- @x ~ y@.
@@ -970,7 +903,7 @@ addCoreCt nabla x e = do
       core_expr x e
       pure x
 
-    -- | Look at @let x = K taus theta es@ and generate the following
+    -- Look at @let x = K taus theta es@ and generate the following
     -- constraints (assuming universals were dropped from @taus@ before):
     --   1. @x ≁ ⊥@ if 'K' is not a Newtype constructor.
     --   2. @a_1 ~ tau_1, ..., a_n ~ tau_n@ for fresh @a_i@
@@ -997,7 +930,7 @@ addCoreCt nabla x e = do
       -- 4. @x ~ K as ys@
       pm_alt_con_app x (PmAltConLike (RealDataCon dc)) ex_tvs arg_ids
 
-    -- | Adds a literal constraint, i.e. @x ~ 42@.
+    -- Adds a literal constraint, i.e. @x ~ 42@.
     -- Also we assume that literal expressions won't diverge, so this
     -- will add a @x ~/ ⊥@ constraint.
     pm_lit :: Id -> PmLit -> StateT Nabla (MaybeT DsM) ()
@@ -1005,7 +938,7 @@ addCoreCt nabla x e = do
       modifyT $ \nabla -> addNotBotCt nabla x
       pm_alt_con_app x (PmAltLit lit) [] []
 
-    -- | Adds the given constructor application as a solution for @x@.
+    -- Adds the given constructor application as a solution for @x@.
     pm_alt_con_app :: Id -> PmAltCon -> [TyVar] -> [Id] -> StateT Nabla (MaybeT DsM) ()
     pm_alt_con_app x con tvs args = modifyT $ \nabla -> addConCt nabla x con tvs args
 
@@ -1073,7 +1006,7 @@ The term oracle state is never obviously (i.e., without consulting the type
 oracle or doing inhabitation testing) contradictory. This implies a few
 invariants:
 * Whenever vi_pos overlaps with vi_neg according to 'eqPmAltCon', we refute.
-  This is implied by the Note [Pos/Neg invariant].
+  This is implied by the Note [The Pos/Neg invariant].
 * Whenever vi_neg subsumes a COMPLETE set, we refute. We consult vi_rcm to
   detect this, but we could just compare whole COMPLETE sets to vi_neg every
   time, if it weren't for performance.
@@ -1262,7 +1195,7 @@ traverseAll f ts@TmSt{ts_facts = env} = do
 -- The \(∇ ⊢ x inh\) judgment form in Figure 8 of the LYG paper.
 inhabitationTest :: Int -> TyState -> Nabla -> MaybeT DsM Nabla
 inhabitationTest 0     _         nabla             = pure nabla
-inhabitationTest fuel  old_ty_st nabla@MkNabla{ nabla_tm_st = ts } = do
+inhabitationTest fuel  old_ty_st nabla@MkNabla{ nabla_tm_st = ts } = {-# SCC "inhabitationTest" #-} do
   -- lift $ tracePm "inhabitation test" $ vcat
   --   [ ppr fuel
   --   , ppr old_ty_st
@@ -1315,11 +1248,12 @@ varNeedsTesting old_ty_st MkNabla{nabla_ty_st=new_ty_st} vi = do
 --     remain that do not statisfy it.  This lazy approach just
 --     avoids doing unnecessary work.
 instantiate :: Int -> Nabla -> VarInfo -> MaybeT DsM VarInfo
-instantiate fuel nabla vi = instBot fuel nabla vi <|> instCompleteSets fuel nabla vi
+instantiate fuel nabla vi = {-# SCC "instantiate" #-}
+  (instBot fuel nabla vi <|> instCompleteSets fuel nabla vi)
 
 -- | The \(⊢_{Bot}\) rule from the paper
 instBot :: Int -> Nabla -> VarInfo -> MaybeT DsM VarInfo
-instBot _fuel nabla vi = do
+instBot _fuel nabla vi = {-# SCC "instBot" #-} do
   _nabla' <- addBotCt nabla (vi_id vi)
   pure vi
 
@@ -1353,7 +1287,7 @@ splitReprTyConApp_maybe env ty =
 -- where all the attempted ConLike instantiations have been purged from the
 -- 'ResidualCompleteMatches', which functions as a cache.
 instCompleteSets :: Int -> Nabla -> VarInfo -> MaybeT DsM VarInfo
-instCompleteSets fuel nabla vi = do
+instCompleteSets fuel nabla vi = {-# SCC "instCompleteSets" #-} do
   let x = vi_id vi
   (rcm, nabla) <- lift (addNormalisedTypeMatches nabla x)
   nabla <- foldM (\nabla cls -> instCompleteSet fuel nabla x cls) nabla (getRcm rcm)
@@ -1384,7 +1318,7 @@ instCompleteSet fuel nabla x cs
   | not (completeMatchAppliesAtType (varType x) cs)
   = pure nabla
   | otherwise
-  = go nabla (sorted_candidates cs)
+  = {-# SCC "instCompleteSet" #-} go nabla (sorted_candidates cs)
   where
     vi = lookupVarInfo (nabla_tm_st nabla) x
 
@@ -1423,23 +1357,26 @@ isDataConTriviallyInhabited :: DataCon -> Bool
 isDataConTriviallyInhabited dc
   | isTyConTriviallyInhabited (dataConTyCon dc) = True
 isDataConTriviallyInhabited dc =
-  null (dataConTheta dc) &&         -- (1)
-  null (dataConImplBangs dc) &&     -- (2)
-  null (dataConUnliftedFieldTys dc) -- (3)
+  null (dataConTheta dc) &&                -- (1)
+  null (dataConImplBangs dc) &&            -- (2)
+  null (dataConMightBeUnliftedFieldTys dc) -- (3)
 
-dataConUnliftedFieldTys :: DataCon -> [Type]
-dataConUnliftedFieldTys =
-  -- A levity polymorphic field requires an inhabitation test, hence compare to
-  -- @Just True@
-  filter ((== Just True) . isLiftedType_maybe) . map scaledThing . dataConOrigArgTys
+dataConMightBeUnliftedFieldTys :: DataCon -> [Type]
+dataConMightBeUnliftedFieldTys =
+  filter mightBeUnliftedType . map scaledThing . dataConOrigArgTys
 
 isTyConTriviallyInhabited :: TyCon -> Bool
-isTyConTriviallyInhabited tc = elementOfUniqSet tc triviallyInhabitedTyCons
+isTyConTriviallyInhabited tc = elementOfUniqSet (getUnique tc) triviallyInhabitedTyConKeys
 
 -- | All these types are trivially inhabited
-triviallyInhabitedTyCons :: UniqSet TyCon
-triviallyInhabitedTyCons = mkUniqSet [
-    charTyCon, doubleTyCon, floatTyCon, intTyCon, wordTyCon, word8TyCon
+triviallyInhabitedTyConKeys :: UniqSet Unique
+triviallyInhabitedTyConKeys = mkUniqSet [
+    charTyConKey, doubleTyConKey, floatTyConKey,
+    intTyConKey, int8TyConKey, int16TyConKey, int32TyConKey, int64TyConKey,
+    intPrimTyConKey, int8PrimTyConKey, int16PrimTyConKey, int32PrimTyConKey, int64PrimTyConKey,
+    wordTyConKey, word8TyConKey, word16TyConKey, word32TyConKey, word64TyConKey,
+    wordPrimTyConKey, word8PrimTyConKey, word16PrimTyConKey, word32PrimTyConKey, word64PrimTyConKey,
+    trTyConTyConKey
   ]
 
 compareConLikeTestability :: ConLike -> ConLike -> Ordering
@@ -1464,16 +1401,19 @@ compareConLikeTestability (RealDataCon a) (RealDataCon b) = mconcat
     -- the unlikely bogus case of an unlifted field that has a bang.
     unlifted_or_strict_fields :: DataCon -> Int
     unlifted_or_strict_fields dc = fast_length (dataConImplBangs dc)
-                                 + fast_length (dataConUnliftedFieldTys dc)
+                                 + fast_length (dataConMightBeUnliftedFieldTys dc)
 
 -- | @instCon fuel nabla (x::match_ty) K@ tries to instantiate @x@ to @K@ by
 -- adding the proper constructor constraint.
 --
 -- See Note [Instantiating a ConLike].
 instCon :: Int -> Nabla -> Id -> ConLike -> MaybeT DsM Nabla
-instCon fuel nabla@MkNabla{nabla_ty_st = ty_st} x con = MaybeT $ do
+instCon fuel nabla@MkNabla{nabla_ty_st = ty_st} x con = {-# SCC "instCon" #-} MaybeT $ do
+  let hdr what = "instCon " ++ show fuel ++ " " ++ what
   env <- dsGetFamInstEnvs
   let match_ty = idType x
+  tracePm (hdr "{") $
+    ppr con <+> text "... <-" <+> ppr x <+> dcolon <+> ppr match_ty
   norm_match_ty <- normaliseSourceTypeWHNF ty_st match_ty
   mb_sigma_univ <- matchConLikeResTy env ty_st norm_match_ty con
   case mb_sigma_univ of
@@ -1490,28 +1430,45 @@ instCon fuel nabla@MkNabla{nabla_ty_st = ty_st} x con = MaybeT $ do
       -- (4) Instantiate fresh term variables as arguments to the constructor
       let field_tys' = substTys sigma_ex $ map scaledThing field_tys
       arg_ids <- mapM mkPmId field_tys'
-      tracePm "instCon" $ vcat
+      tracePm (hdr "(cts)") $ vcat
         [ ppr x <+> dcolon <+> ppr match_ty
         , text "In WHNF:" <+> ppr (isSourceTypeInWHNF match_ty) <+> ppr norm_match_ty
         , ppr con <+> dcolon <+> text "... ->" <+> ppr _con_res_ty
         , ppr (map (\tv -> ppr tv <+> char '↦' <+> ppr (substTyVar sigma_univ tv)) _univ_tvs)
         , ppr gammas
         , ppr (map (\x -> ppr x <+> dcolon <+> ppr (idType x)) arg_ids)
-        , ppr fuel
         ]
       -- (5) Finally add the new constructor constraint
       runMaybeT $ do
         -- Case (2) of Note [Strict fields and variables of unlifted type]
         let alt = PmAltConLike con
-        nabla' <- addPhiTmCt nabla (PhiConCt x alt ex_tvs gammas arg_ids)
         let branching_factor = length $ filterUnliftedFields alt arg_ids
+        let ct = PhiConCt x alt ex_tvs gammas arg_ids
+        nabla1 <- traceWhenFailPm (hdr "(ct unsatisfiable) }") (ppr ct) $
+                  addPhiTmCt nabla ct
         -- See Note [Fuel for the inhabitation test]
         let new_fuel
               | branching_factor <= 1 = fuel
               | otherwise             = min fuel 2
-        inhabitationTest new_fuel (nabla_ty_st nabla) nabla'
-    Nothing -> pure (Just nabla) -- Matching against match_ty failed. Inhabited!
-                                 -- See Note [Instantiating a ConLike].
+        lift $ tracePm (hdr "(ct satisfiable)") $ vcat
+          [ ppr ct
+          , ppr x <+> dcolon <+> ppr match_ty
+          , text "In WHNF:" <+> ppr (isSourceTypeInWHNF match_ty) <+> ppr norm_match_ty
+          , ppr con <+> dcolon <+> text "... ->" <+> ppr _con_res_ty
+          , ppr (map (\tv -> ppr tv <+> char '↦' <+> ppr (substTyVar sigma_univ tv)) _univ_tvs)
+          , ppr gammas
+          , ppr (map (\x -> ppr x <+> dcolon <+> ppr (idType x)) arg_ids)
+          , ppr branching_factor
+          , ppr new_fuel
+          ]
+        nabla2 <- traceWhenFailPm (hdr "(inh test failed) }") (ppr nabla1) $
+                  inhabitationTest new_fuel (nabla_ty_st nabla) nabla1
+        lift $ tracePm (hdr "(inh test succeeded) }") (ppr nabla2)
+        pure nabla2
+    Nothing -> do
+      tracePm (hdr "(match_ty not instance of res_ty) }") empty
+      pure (Just nabla) -- Matching against match_ty failed. Inhabited!
+                        -- See Note [Instantiating a ConLike].
 
 -- | @matchConLikeResTy _ _ ty K@ tries to match @ty@ against the result
 -- type of @K@, @res_ty@. It returns a substitution @s@ for @K@'s universal
@@ -1526,7 +1483,7 @@ matchConLikeResTy env _              ty (RealDataCon dc) = pure $ do
   if rep_tc == dataConTyCon dc
     then Just (zipTCvSubst (dataConUnivTyVars dc) tc_args)
     else Nothing
-matchConLikeResTy _   (TySt _ inert) ty (PatSynCon ps) = runMaybeT $ do
+matchConLikeResTy _   (TySt _ inert) ty (PatSynCon ps) = {-# SCC "matchConLikeResTy" #-} runMaybeT $ do
   let (univ_tvs,req_theta,_,_,_,con_res_ty) = patSynSig ps
   subst <- MaybeT $ pure $ tcMatchTy con_res_ty ty
   guard $ all (`elemTCvSubst` subst) univ_tvs -- See the Note about T11336b
@@ -1539,9 +1496,9 @@ matchConLikeResTy _   (TySt _ inert) ty (PatSynCon ps) = runMaybeT $ do
         then pure subst
         else mzero
 
-{- Note [Soundness and completeness]
+{- Note [Soundness and Completeness]
 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-Soundness and completeness of the pattern-match checker depends entirely on the
+Soundness and completeness of the pattern-match checker depend entirely on the
 soundness and completeness of the inhabitation test.
 
 Achieving both soundness and completeness at the same time is undecidable.
@@ -1660,7 +1617,7 @@ instantiation and requires an inductive argument, which `inhabitationTest`
 currently isn't equipped to do.
 
 In order to prevent endless instantiation attempts in @inhabitationTest@, we
-use the fuel as an upper bound such attempts.
+use the fuel as an upper bound on such attempts.
 
 The same problem occurs with recursive newtypes, like in the following code:
 
@@ -1798,8 +1755,8 @@ See Note [Matching against a ConLike result type].
 If matching /fails/, it trivially (and conservatively) reports "inhabited" by
 returning the unrefined input Nabla. After all, the match might have failed due
 to incomplete type information in Nabla.
-(Type refinement from unpacking GADT constructors might monomorphise `match_ty`
-so much that `res_ty` ultimately subsumes it.)
+(Later type refinement from unpacking GADT constructors might monomorphise
+`match_ty` so much that `res_ty` ultimately subsumes it.)
 
 If matching /succeeds/, we get a substitution σ for the (universal)
 tyvars `us`. After applying σ, we get
@@ -1833,38 +1790,53 @@ compatible.
 -- This is important for warnings. Roughly corresponds to G in Figure 6 of the
 -- LYG paper, with a few tweaks for better warning messages.
 
+-- | See Note [Case split inhabiting patterns]
+data GenerateInhabitingPatternsMode
+  = CaseSplitTopLevel
+  | MinimalCover
+  deriving (Eq, Show)
+
+instance Outputable GenerateInhabitingPatternsMode where
+  ppr = text . show
+
 -- | @generateInhabitingPatterns vs n nabla@ returns a list of at most @n@ (but
 -- perhaps empty) refinements of @nabla@ that represent inhabited patterns.
 -- Negative information is only retained if literals are involved or for
 -- recursive GADTs.
-generateInhabitingPatterns :: [Id] -> Int -> Nabla -> DsM [Nabla]
+generateInhabitingPatterns :: GenerateInhabitingPatternsMode -> [Id] -> Int -> Nabla -> DsM [Nabla]
 -- See Note [Why inhabitationTest doesn't call generateInhabitingPatterns]
-generateInhabitingPatterns _      0 _     = pure []
-generateInhabitingPatterns []     _ nabla = pure [nabla]
-generateInhabitingPatterns (x:xs) n nabla = do
-  tracePm "generateInhabitingPatterns" (ppr n <+> ppr (x:xs) $$ ppr nabla)
+generateInhabitingPatterns _    _      0 _     = pure []
+generateInhabitingPatterns _    []     _ nabla = pure [nabla]
+generateInhabitingPatterns mode (x:xs) n nabla = do
+  tracePm "generateInhabitingPatterns" (ppr mode <+> ppr n <+> ppr (x:xs) $$ ppr nabla)
   let VI _ pos neg _ _ = lookupVarInfo (nabla_tm_st nabla) x
   case pos of
     _:_ -> do
-      -- All solutions must be valid at once. Try to find candidates for their
-      -- fields. Example:
-      --   f x@(Just _) True = case x of SomePatSyn _ -> ()
-      -- after this clause, we want to report that
-      --   * @f Nothing _@ is uncovered
-      --   * @f x False@ is uncovered
-      -- where @x@ will have two possibly compatible solutions, @Just y@ for
-      -- some @y@ and @SomePatSyn z@ for some @z@. We must find evidence for @y@
-      -- and @z@ that is valid at the same time. These constitute arg_vas below.
+      -- Example for multiple solutions (must involve a PatSyn):
+      --   f x@(Just _) True | SomePatSyn _ <- x = ...
+      -- within the RHS, we know that
+      --   * @x ~ Just y@ for some @y@
+      --   * @x ~ SomePatSyn z@ for some @z@
+      -- and both conditions have to hold /at the same time/. Hence we must
+      -- find evidence for @y@ and @z@ that is valid at the same time. These
+      -- constitute arg_vas below.
       let arg_vas = concatMap paca_ids pos
-      generateInhabitingPatterns (arg_vas ++ xs) n nabla
+      generateInhabitingPatterns mode (arg_vas ++ xs) n nabla
     []
       -- When there are literals involved, just print negative info
       -- instead of listing missed constructors
       | notNull [ l | PmAltLit l <- pmAltConSetElems neg ]
-      -> generateInhabitingPatterns xs n nabla
-    [] -> try_instantiate x xs n nabla
+      -> generateInhabitingPatterns mode xs n nabla
+      -- When some constructors are impossible or when we don't care for a
+      -- minimal cover (Note [Case split inhabiting patterns]), do a case split.
+      | not (isEmptyPmAltConSet neg) || mode == CaseSplitTopLevel
+      -> try_instantiate x xs n nabla
+      -- Else don't do a case split on this var, just print a wildcard.
+      -- But try splitting for the remaining vars, of course
+      | otherwise
+      -> generateInhabitingPatterns mode xs n nabla
   where
-    -- | Tries to instantiate a variable by possibly following the chain of
+    -- Tries to instantiate a variable by possibly following the chain of
     -- newtypes and then instantiating to all ConLikes of the wrapped type's
     -- minimal residual COMPLETE set.
     try_instantiate :: Id -> [Id] -> Int -> Nabla -> DsM [Nabla]
@@ -1886,17 +1858,17 @@ generateInhabitingPatterns (x:xs) n nabla = do
           case NE.nonEmpty (uniqDSetToList . cmConLikes <$> clss) of
             Nothing ->
               -- No COMPLETE sets ==> inhabited
-              generateInhabitingPatterns xs n newty_nabla
+              generateInhabitingPatterns mode xs n newty_nabla
             Just clss -> do
               -- Try each COMPLETE set, pick the one with the smallest number of
               -- inhabitants
               nablass' <- forM clss (instantiate_cons y rep_ty xs n newty_nabla)
               let nablas' = minimumBy (comparing length) nablass'
               if null nablas' && vi_bot vi /= IsNotBot
-                then generateInhabitingPatterns xs n newty_nabla -- bot is still possible. Display a wildcard!
+                then generateInhabitingPatterns mode xs n newty_nabla -- bot is still possible. Display a wildcard!
                 else pure nablas'
 
-    -- | Instantiates a chain of newtypes, beginning at @x@.
+    -- Instantiates a chain of newtypes, beginning at @x@.
     -- Turns @x nabla [T,U,V]@ to @(y, nabla')@, where @nabla'@ we has the fact
     -- @x ~ T (U (V y))@.
     instantiate_newtype_chain :: Id -> Nabla -> [(Type, DataCon, Type)] -> MaybeT DsM (Id, Nabla)
@@ -1914,7 +1886,7 @@ generateInhabitingPatterns (x:xs) n nabla = do
     instantiate_cons _ ty xs n nabla _
       -- We don't want to expose users to GHC-specific constructors for Int etc.
       | fmap (isTyConTriviallyInhabited . fst) (splitTyConApp_maybe ty) == Just True
-      = generateInhabitingPatterns xs n nabla
+      = generateInhabitingPatterns mode xs n nabla
     instantiate_cons x ty xs n nabla (cl:cls) = do
       -- The following line is where we call out to the inhabitationTest!
       mb_nabla <- runMaybeT $ instCon 4 nabla x cl
@@ -1929,7 +1901,7 @@ generateInhabitingPatterns (x:xs) n nabla = do
         -- NB: We don't prepend arg_vars as we don't have any evidence on
         -- them and we only want to split once on a data type. They are
         -- inhabited, otherwise the inhabitation test would have refuted.
-        Just nabla' -> generateInhabitingPatterns xs n nabla'
+        Just nabla' -> generateInhabitingPatterns mode xs n nabla'
       other_cons_nablas <- instantiate_cons x ty xs (n - length con_nablas) nabla cls
       pure (con_nablas ++ other_cons_nablas)
 
@@ -1955,7 +1927,7 @@ pickApplicableCompleteSets ty_st ty rcm = do
   return applicable_cms
 
 {- Note [Why inhabitationTest doesn't call generateInhabitingPatterns]
-~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 Why can't we define `inhabitationTest` (IT) in terms of
 `generateInhabitingPatterns` (GIP) as
 
@@ -1975,4 +1947,36 @@ efficient IT on huge enumerations.
 But we still need GIP to produce the Nablas as proxies for
 uncovered patterns that we display warnings for. It's fine to pay this price
 once at the end, but IT is called far more often than that.
+
+Note [Case split inhabiting patterns]
+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+When we have an -XEmptyCase,
+```hs
+f :: Maybe a -> ()
+f x = case x of {}
+```
+we want to show the different missing patterns {'Just _', 'Nothing'} to the user
+instead of an uninformative wildcard pattern '_'.
+
+But when we have a match as part of a function definition, such as
+```hs
+g :: (Bool, [a]) -> ()
+g (_, []) = ()
+```
+we established in #20642 that we *do not* want to report P1={'(False, (_:_))',
+'(True, (_:_))'} when we could just give the singleton set P2={'(_, (_:_))'}!
+The latter set M is a "minimal cover" of the space of missing patterns S, in
+that
+
+1. The union of M is *exactly* S
+2. The size of M is minimal under all such candidates.
+
+In terms of 'g', P2 satisfies both (1) and (2). While P1 satisfies (1), it does
+not satisfy (2). The set {'_'} satisfies (2), but it does not satisfy (1) as
+it also covers the pattern '(_,[])' which is not part of the space of missing
+patterns of 'g', so it would be a bad (too conservative) suggestion.
+
+Note that for -XEmptyCase, we don't want to emit a minimal cover. We arrange
+that by passing 'CaseSplitTopLevel' to 'generateInhabitingPatterns'. We detect
+the -XEmptyCase case in 'reportWarnings' by looking for 'ReportEmptyCase'.
 -}
