@@ -24,8 +24,11 @@ import Test.Cabal.Prelude
 -- Recording is turned off because versionedlib will or will not
 -- be installed depending on if we're on Linux or not.
 main = setupAndCabalTest . recordMode DoNotRecord $ do
-    -- Foreign libraries don't work with GHC 7.6 and earlier
-    skipUnlessGhcVersion ">= 7.8"
+  -- Foreign libraries don't work with GHC 7.6 and earlier
+  skipUnlessGhcVersion ">= 7.8"
+  osx <- isOSX
+  ghc80 <- isGhcVersion "== 8.0.2"
+  expectBrokenIf (osx && ghc80) 7989 $
     withPackageDb $ do
         setup_install []
         setup "copy" [] -- regression test #4156
