@@ -152,7 +152,7 @@ instance Show ThreadId where
 showThreadId :: ThreadId -> String
 showThreadId = show
 
-foreign import ccall unsafe "rts_getThreadId" getThreadId :: ThreadId# -> CInt
+foreign import ccall unsafe "rts_getThreadId" getThreadId :: ThreadId# -> CULLong
 
 id2TSO :: ThreadId -> ThreadId#
 id2TSO (ThreadId t) = t
@@ -260,10 +260,15 @@ The newly created thread has an exception handler that discards the
 exceptions 'BlockedIndefinitelyOnMVar', 'BlockedIndefinitelyOnSTM', and
 'ThreadKilled', and passes all other exceptions to the uncaught
 exception handler.
+
+WARNING: Exceptions in the new thread will not be rethrown in the thread that
+created it. This means that you might be completely unaware of the problem
+if/when this happens.  You may want to use the
+<hackage.haskell.org/package/async async> library instead.
 -}
 forkIO :: IO () -> IO ThreadId
 forkIO action = IO $ \ s ->
-   case (fork# action_plus s) of (# s1, tid #) -> (# s1, ThreadId tid #)
+   case (fork# (unIO action_plus) s) of (# s1, tid #) -> (# s1, ThreadId tid #)
  where
   -- We must use 'catch' rather than 'catchException' because the action
   -- could be bottom. #13330
@@ -313,7 +318,7 @@ is recommended).
 -}
 forkOn :: Int -> IO () -> IO ThreadId
 forkOn (I# cpu) action = IO $ \ s ->
-   case (forkOn# cpu action_plus s) of (# s1, tid #) -> (# s1, ThreadId tid #)
+   case (forkOn# cpu (unIO action_plus) s) of (# s1, tid #) -> (# s1, ThreadId tid #)
  where
   -- We must use 'catch' rather than 'catchException' because the action
   -- could be bottom. #13330
@@ -568,7 +573,7 @@ threadStatus (ThreadId t) = IO $ \s ->
    case threadStatus# t s of
     (# s', stat, _cap, _locked #) -> (# s', mk_stat (I# stat) #)
    where
-        -- NB. keep these in sync with includes/rts/Constants.h
+        -- NB. keep these in sync with rts/include/rts/Constants.h
      mk_stat 0  = ThreadRunning
      mk_stat 1  = ThreadBlocked BlockedOnMVar
      mk_stat 2  = ThreadBlocked BlockedOnBlackHole
@@ -621,12 +626,12 @@ data PrimMVar
 -- @hs_try_putmvar()@.  The RTS wants a 'StablePtr' to the underlying
 -- 'MVar#', but a 'StablePtr#' can only refer to lifted types, so we
 -- have to cheat by coercing.
-newStablePtrPrimMVar :: MVar () -> IO (StablePtr PrimMVar)
+newStablePtrPrimMVar :: MVar a -> IO (StablePtr PrimMVar)
 newStablePtrPrimMVar (MVar m) = IO $ \s0 ->
   case makeStablePtr# (unsafeCoerce# m :: PrimMVar) s0 of
-    -- Coerce unlifted  m :: MVar# RealWorld ()
+    -- Coerce unlifted  m :: MVar# RealWorld a
     --     to lifted    PrimMVar
-    -- apparently because mkStablePtr is not levity-polymorphic
+    -- apparently because mkStablePtr is not representation-polymorphic
     (# s1, sp #) -> (# s1, StablePtr sp #)
 
 -----------------------------------------------------------------------------
@@ -662,6 +667,14 @@ instance  Monad STM  where
     m >>= k     = bindSTM m k
     (>>) = (*>)
 
+-- | @since 4.17.0.0
+instance Semigroup a => Semigroup (STM a) where
+    (<>) = liftA2 (<>)
+
+-- | @since 4.17.0.0
+instance Monoid a => Monoid (STM a) where
+    mempty = pure mempty
+
 bindSTM :: STM a -> (a -> STM b) -> STM b
 bindSTM (STM m) k = STM ( \s ->
   case m s of
