@@ -34,7 +34,6 @@ import GHC.IO.Encoding
 module Properties.ByteStringLazy (tests) where
 #define BYTESTRING_TYPE B.ByteString
 import qualified Data.ByteString.Lazy as B
-import qualified Data.ByteString.Lazy.Internal as B (invariant)
 #endif
 
 import Data.Word
@@ -48,7 +47,6 @@ import qualified Data.ByteString.Char8 as B
 #else
 module Properties.ByteStringLazyChar8 (tests) where
 import qualified Data.ByteString.Lazy.Char8 as B
-import qualified Data.ByteString.Lazy.Internal as B (invariant)
 #define BYTESTRING_TYPE B.ByteString
 #endif
 
@@ -56,10 +54,12 @@ import Text.Read
 
 #endif
 
+import Prelude hiding (head, tail)
 import Control.Arrow
 import Data.Char
 import Data.Foldable
 import qualified Data.List as List
+import qualified Data.List.NonEmpty as NE
 import Data.Semigroup
 import Data.String
 import Data.Tuple
@@ -102,8 +102,9 @@ tests =
     -- but if it is ASCII, we should not generate Unicode filenames.
     enc <- getFileSystemEncoding
     pure $ case textEncodingName enc of
-      "ASCII" -> property (prop . getASCIIString)
-      _       -> property prop
+      "ASCII"          -> property (prop . getASCIIString)
+      "ANSI_X3.4-1968" -> property (prop . getASCIIString)
+      _                -> property prop
 #endif
 
   , testProperty "==" $
@@ -183,6 +184,10 @@ tests =
     \x -> map B.unpack (B.inits x) === List.inits (B.unpack x)
   , testProperty "tails" $
     \x -> map B.unpack (B.tails x) === List.tails (B.unpack x)
+  , testProperty "initsNE" $
+    \x -> NE.map B.unpack (B.initsNE x) === NE.inits (B.unpack x)
+  , testProperty "tailsNE" $
+    \x -> NE.map B.unpack (B.tailsNE x) === NE.tails (B.unpack x)
 #endif
   , testProperty "all" $
     \f x -> B.all f x === all f (B.unpack x)
@@ -299,8 +304,6 @@ tests =
     \f x -> B.takeWhileEnd f x === B.reverse (B.takeWhile f (B.reverse x))
 
 #ifdef BYTESTRING_LAZY
-  , testProperty "invariant" $
-    \x -> B.invariant x
   , testProperty "fromChunks . toChunks" $
     \x -> B.fromChunks (B.toChunks x) === x
   , testProperty "toChunks . fromChunks" $
@@ -427,11 +430,15 @@ tests =
     \x -> let n = 2^31 in (B.unpack *** B.unpack) (B.splitAt n x) === List.genericSplitAt n (B.unpack x)
 
   , testProperty "head" $
-    \x -> not (B.null x) ==> B.head x === head (B.unpack x)
+    \x -> case B.unpack x of
+      []     -> property True
+      hd : _ -> B.head x === hd
   , testProperty "last" $
     \x -> not (B.null x) ==> B.last x === last (B.unpack x)
   , testProperty "tail" $
-    \x -> not (B.null x) ==> B.unpack (B.tail x) === tail (B.unpack x)
+    \x -> case B.unpack x of
+      []     -> property True
+      _ : tl -> B.unpack (B.tail x) === tl
   , testProperty "tail length" $
     \x -> not (B.null x) ==> B.length x === 1 + B.length (B.tail x)
   , testProperty "init" $
