@@ -772,7 +772,7 @@ void rts_resume (PauseToken *pauseToken)
 
     // releaseAllCapabilities will not block because the current task owns all
     // capabilities.
-    releaseAllCapabilities(n_capabilities, NULL, task);
+    releaseAllCapabilities(getNumCapabilities(), NULL, task);
     exitMyTask();
     stgFree(pauseToken);
 }
@@ -810,9 +810,9 @@ static void assert_isPausedOnMyTask(const char *functionName)
     }
 
     // Check that we own all capabilities.
-    for (unsigned int i = 0; i < n_capabilities; i++)
+    for (unsigned int i = 0; i < getNumCapabilities(); i++)
     {
-        Capability *cap = capabilities[i];
+        Capability *cap = getCapability(i);
         if (cap->running_task != task)
         {
             errorBelch (
@@ -882,7 +882,7 @@ rts_resume (PauseToken *pauseToken STG_UNUSED)
     stg_exit(EXIT_FAILURE);
 }
 
-bool rts_isPaused()
+bool rts_isPaused(void)
 {
     errorBelch("Warning: Pausing/Resuming the RTS is only possible for "
                "multithreaded RTS.");
@@ -947,7 +947,7 @@ void hs_try_putmvar (/* in */ int capability,
             capability = 0;
         }
     }
-    cap = capabilities[capability % enabled_capabilities];
+    cap = getCapability(capability % enabled_capabilities);
 
 #if !defined(THREADED_RTS)
 
