@@ -81,7 +81,6 @@ RTS_RET(stg_restore_cccs_eval);
 /* Magic glue code for when compiled code returns a value in R1/F1/D1
    or a VoidRep to the interpreter. */
 RTS_RET(stg_ctoi_R1p);
-RTS_RET(stg_ctoi_R1unpt);
 RTS_RET(stg_ctoi_R1n);
 RTS_RET(stg_ctoi_F1);
 RTS_RET(stg_ctoi_D1);
@@ -159,6 +158,7 @@ RTS_RET(stg_ctoi_t60);
 RTS_RET(stg_ctoi_t61);
 RTS_RET(stg_ctoi_t62);
 
+RTS_RET(stg_primcall);
 RTS_RET(stg_apply_interp);
 
 RTS_ENTRY(stg_IND);
