@@ -16,7 +16,7 @@
 # Add missing targets to the list below to have them included in
 # llvm-targets file.
 #
-# See Note [LLVM Configuration] in SysTools for the whole story regarding LLVM
+# See Note [LLVM configuration] in GHC.SysTools for the whole story regarding LLVM
 # configuration data.
 
 # Target sets for which to generate the llvm-targets file
@@ -107,6 +107,19 @@ TARGETS=(
     "armv6-unknown-freebsd-gnueabihf"
     "armv7-unknown-freebsd-gnueabihf"
 
+    #########################
+    # NetBSD
+    #########################
+
+    "aarch64-unknown-netbsd"
+
+    #########################
+    # OpenBSD
+    #########################
+
+    "x86_64-unknown-openbsd"
+    "i386-unknown-openbsd"
+
     #########################
     # Other
     #########################
